@@ -2,7 +2,7 @@
 # @Author: Anderson
 # @Date:   2019-11-11 17:42:18
 # @Last Modified by:   ander
-# @Last Modified time: 2020-05-19 16:55:43
+# @Last Modified time: 2020-05-26 11:51:18
 from openpyxl import Workbook, load_workbook
 from docx import Document
 import requests
@@ -18,9 +18,13 @@ import csv
 from pyecharts import options as opts
 from pyecharts.charts import Bar
 from pyecharts.charts import WordCloud
+from pyecharts.charts import Map3D
+from pyecharts.globals import ChartType
+from pyecharts.commons.utils import JsCode
 from copy import copy
 from collections import Counter
 from .stop_words import stop_words
+from .data import provinces_coordinates
 
 
 def validate_title(title):
@@ -189,7 +193,7 @@ class ExcelBot(object):
 	def get_row(self, row):
 		data = []
 		if isinstance(row, int):
-			for cell in self.sheet[row]:
+			for cell in self.sheet[row + 1]:
 				data.append(cell.value)
 
 		return data
@@ -212,8 +216,16 @@ class ExcelBot(object):
 		self.sheet = self.workbook.create_sheet('sheet1')
 
 	def open(self, filename):
-		self.workbook = load_workbook(filename=filename)
-		self.sheet = self.workbook.active
+		if filename.endswith('.csv'):
+			self.workbook = Workbook()
+			self.sheet = self.workbook.active
+			with open(filename) as f:
+				reader = csv.reader(f)
+				for row in reader:
+					self.sheet.append(row)
+		else:
+			self.workbook = load_workbook(filename=filename)
+			self.sheet = self.workbook.active
 
 	def save(self, filename, ext='xlsx'):
 		self.filename = filename
@@ -245,6 +257,10 @@ class WordBot(object):
 		else:
 			return ''
 
+	@property
+	def paragraphs(self):
+		return [str(p.text) for p in self.doc.paragraphs]
+
 	def clear(self):
 		self.doc = Document()
 
@@ -252,6 +268,8 @@ class WordBot(object):
 		self.doc = Document(filename)
 
 	def save(self, filename):
+		if not filename:
+			filename = 'tmp'
 		self.doc.save(f'{filename}.docx')
 
 
@@ -328,6 +346,65 @@ class DataAnalysisBot(object):
 			xaxis_opts=opts.AxisOpts(axislabel_opts=opts.LabelOpts(rotate=10)))
 		bar.render('bar.html')
 
+	def generate_3d_map(self, provinces_data):
+		map_3d = Map3D(
+			init_opts=opts.InitOpts(
+				width="1280px",
+				height="720px"))
+		map_3d.add_schema(
+			itemstyle_opts=opts.ItemStyleOpts(
+				color="rgb(242, 105, 92)",
+				opacity=1,
+				border_width=0.8,
+				border_color="rgb(242, 181, 145)",
+			),
+			map3d_label=opts.Map3DLabelOpts(
+				is_show=False,
+				formatter=JsCode("function(data){return data.name + " " + data.value[2];}"),
+			),
+			emphasis_label_opts=opts.LabelOpts(
+				is_show=False,
+				color="#fff",
+				font_size=10,
+				background_color="rgba(242, 149, 128, 0)",
+			),
+			light_opts=opts.Map3DLightOpts(
+				main_color="#fff",
+				main_intensity=1.2,
+				main_shadow_quality="high",
+				is_main_shadow=True,
+				main_beta=10,
+				ambient_intensity=0.3,
+			),
+			is_show_ground=True,
+			ground_color="#F2B591",
+		)
+
+		map_3d_data = []
+		for item in provinces_data.items():
+			province_data = (
+				item[0],
+				[*provinces_coordinates[item[0]], item[1]]
+			)
+			map_3d_data.append(province_data)
+
+		map_3d.add(
+			series_name="bar3D",
+			data_pair=map_3d_data,
+			type_=ChartType.BAR3D,
+			bar_size=1,
+			itemstyle_opts=opts.ItemStyleOpts(
+				color="rgb(173, 212, 217)"
+			),
+			shading="realistic",
+			label_opts=opts.LabelOpts(
+				is_show=False,
+				formatter=JsCode("function(data){return data.name + ' ' + data.value[2];}"),
+			),
+		)
+		map_3d.set_global_opts(title_opts=opts.TitleOpts(title="Map3D-Bar3D"))
+		map_3d.render("map3d_with_bar3d.html")
+
 
 pachong = wc_bot = web_crawler_bot = WebCrawlerBot()
 biaoge = ec_bot = excel_bot = ExcelBot()
